/*
 * This file is a part of KEMET project (Library).
 * 
 * Copyright (C) 2008 MAUPIN Fabrice
 *
 * This library is free software; 
 * you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; 
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this library; 
 * if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */
package org.kemet.tests.integration.transliteration;

import org.apache.log4j.Logger;
import org.junit.Test;
import org.kemet.impl.grammar.exceptions.KemetAPIException;
import org.kemet.impl.grammar.integration.TransliterationManager;
import org.kemet.tests.TestRoot;

/**
 * 
 * @author fabricemaupin
 * 
 * <p>
 * <b>integration tests : analyze type of translitered sequence - output : list
 * of type for each part of sequence.</b>
 * </p>
 * 
 * @since 0.5
 * 
 * support : uniliteral, biliteral, triliteral, phonetic complement, indicator
 * stating of adverbial predicate proposition (ppa), pronoun, preposition, noun,
 * ideogram
 * 
 * not supported (no transliteration) : determiner
 */
public class TestsToType extends TestRoot {
	static final Logger log = Logger.getLogger("org.kemet.tests");

	/**
	 * <p>
	 * <b>initialize the parameters before to execute the tests.</b>
	 * </p>
	 * 
	 * @since 0.5
	 */
	public void init() {
		// nothing
	}

	/**
	 * <p>
	 * <b>indicator ppa + demonstrative pronoun + biliteral + phonetic
	 * complement + triliteral + phonetic complement.</b>
	 * </p>
	 * 
	 * @since 0.5
	 */
	@Test
	public void test1() {
		// initialize the data tests
		sequence = "jwnnc3cjm33";

		result = new String[] {
				org.kemet.message.Messages.getString("INDICATOR_PPA"),
				org.kemet.message.Messages.getString("DEMONSTRATIVE_PRONOUN"),
				org.kemet.message.Messages.getString("BILITERAL"),
				org.kemet.message.Messages.getString("PC"),
				org.kemet.message.Messages.getString("TRILITERAL"),
				org.kemet.message.Messages.getString("PC") };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_TYPE);

		String[] res;
		try {
			res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>INDICATOR_PPA + uniliteral + uniliteral + uniliteral + biliteral +
	 * biliteral.</b>
	 * </p>
	 * 
	 * @since 0.5
	 */
	@Test
	public void test2() {
		// initialize the data tests
		sequence = "jwnhhnjwn";

		result = new String[] {
				org.kemet.message.Messages.getString("INDICATOR_PPA"),
				org.kemet.message.Messages.getString("UNILITERAL"),
				org.kemet.message.Messages.getString("UNILITERAL"),
				org.kemet.message.Messages.getString("UNILITERAL"),
				org.kemet.message.Messages.getString("BILITERAL"),
				org.kemet.message.Messages.getString("BILITERAL") };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_TYPE);

		String[] res;
		try {
			res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>INDICATOR_PPA + demonstrative pronoun + preposition.</b>
	 * </p>
	 * 
	 * @since 0.5
	 */
	@Test
	public void test3() {
		// initialize the data tests
		sequence = "jwnnn";

		result = new String[] {
				org.kemet.message.Messages.getString("INDICATOR_PPA"),
				org.kemet.message.Messages.getString("DEMONSTRATIVE_PRONOUN"),
				org.kemet.message.Messages.getString("PREPOSITION") };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_TYPE);

		String[] res;
		try {
			res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>INDICATOR_PPA + DEMONSTRATIVE_PRONOUN + DEMONSTRATIVE_PRONOUN.</b>
	 * </p>
	 * 
	 * @since 0.5
	 */
	@Test
	public void test4() {
		// initialize the data tests
		sequence = "jwnnnn";

		result = new String[] {
				org.kemet.message.Messages.getString("INDICATOR_PPA"),
				org.kemet.message.Messages.getString("DEMONSTRATIVE_PRONOUN"),
				org.kemet.message.Messages.getString("DEMONSTRATIVE_PRONOUN") };
		String[] result2 = new String[] { "jw", "nn", "nn" };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_TYPE);

		String[] res;
		try {
			res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}

			for (int i = 0; i < transliteration.getValue().size(); i++) {
				log.info("value(i) = " + transliteration.getValue().get(i));
				assertEquals(result2[i], transliteration.getValue().get(i));
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}

	}

	/**
	 * <p>
	 * <b>INDICATOR_PPA + DEMONSTRATIVE_PRONOUN + DEMONSTRATIVE_PRONOUN +
	 * uniliteral.</b>
	 * </p>
	 * 
	 * @since 0.5
	 */
	@Test
	public void test5() {
		// initialize the data tests
		sequence = "jwnnnnt_";

		result = new String[] {
				org.kemet.message.Messages.getString("INDICATOR_PPA"),
				org.kemet.message.Messages.getString("DEMONSTRATIVE_PRONOUN"),
				org.kemet.message.Messages.getString("DEMONSTRATIVE_PRONOUN"),
				org.kemet.message.Messages.getString("UNILITERAL") };
		String[] result2 = new String[] { "jw", "nn", "nn", "t_" };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_TYPE);

		String[] res;
		try {
			res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}

			for (int i = 0; i < transliteration.getValue().size(); i++) {
				log.info("value(i) = " + transliteration.getValue().get(i));
				assertEquals(result2[i], transliteration.getValue().get(i));
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>INDICATOR_PPA + DEMONSTRATIVE_PRONOUN + DEMONSTRATIVE_PRONOUN +
	 * uniliteral + DEMONSTRATIVE_PRONOUN + uniliteral + uniliteral + uniliteral +
	 * uniliteral + biliteral.</b>
	 * </p>
	 * 
	 * @since 0.5
	 */
	@Test
	public void test6() {
		// initialize the data tests
		sequence = "jwnnnnt_nnkwtp";

		result = new String[] {
				org.kemet.message.Messages.getString("INDICATOR_PPA"),
				org.kemet.message.Messages.getString("DEMONSTRATIVE_PRONOUN"),
				org.kemet.message.Messages.getString("DEMONSTRATIVE_PRONOUN"),
				org.kemet.message.Messages.getString("UNILITERAL"),
				org.kemet.message.Messages.getString("DEMONSTRATIVE_PRONOUN"),
				org.kemet.message.Messages.getString("UNILITERAL"),
				org.kemet.message.Messages.getString("UNILITERAL"),
				org.kemet.message.Messages.getString("PREPOSITION"), };

		String[] result2 = new String[] { "jw", "nn", "nn", "t_", "nn", "k",
				"w", "tp" };

		// simulation of dropped hieroglyphs
		String sourceToCompare = "jw+nn+nn+t_+nn+k+w+tp";

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_TYPE);

		transliteration.setSourceToCompare(sourceToCompare);

		String[] res;
		try {
			res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}

			for (int i = 0; i < transliteration.getValue().size(); i++) {
				log.info("value(i) = " + transliteration.getValue().get(i));
				assertEquals(result2[i], transliteration.getValue().get(i));
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>INDICATOR_PPA + DEMONSTRATIVE_PRONOUN + DEMONSTRATIVE_PRONOUN +
	 * uniliteral + biliteral.</b>
	 * </p>
	 * 
	 * @since 0.5
	 */
	@Test
	public void test7() {
		// initialize the data tests
		sequence = "jwnnnnt_ns";

		result = new String[] {
				org.kemet.message.Messages.getString("INDICATOR_PPA"),
				org.kemet.message.Messages.getString("DEMONSTRATIVE_PRONOUN"),
				org.kemet.message.Messages.getString("DEMONSTRATIVE_PRONOUN"),
				org.kemet.message.Messages.getString("UNILITERAL"),
				org.kemet.message.Messages.getString("BILITERAL") };

		String[] result2 = new String[] { "jw", "nn", "nn", "t_", "ns" };

		// simulation of dropped hieroglyphs
		String sourceToCompare = "jw+nn+nn+t_+ns";

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_TYPE);

		transliteration.setSourceToCompare(sourceToCompare);

		String[] res;
		try {
			res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}

			for (int i = 0; i < transliteration.getValue().size(); i++) {
				log.info("value(i) = " + transliteration.getValue().get(i));
				assertEquals(result2[i], transliteration.getValue().get(i));
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>INDICATOR_PPA + DEMONSTRATIVE_PRONOUN + DEMONSTRATIVE_PRONOUN +
	 * biliteral + biliteral.</b>
	 * </p>
	 * 
	 * @since 0.5
	 */
	@Test
	public void test8() {
		// initialize the data tests
		sequence = "jwnnnnnjk3";

		result = new String[] {
				org.kemet.message.Messages.getString("INDICATOR_PPA"),
				org.kemet.message.Messages.getString("DEMONSTRATIVE_PRONOUN"),
				org.kemet.message.Messages.getString("DEMONSTRATIVE_PRONOUN"),
				org.kemet.message.Messages.getString("BILITERAL"),
				org.kemet.message.Messages.getString("BILITERAL") };

		String[] result2 = new String[] { "jw", "nn", "nn", "nj", "k3" };

		// simulation of dropped hieroglyphs
		String sourceToCompare = "jw+nn+nn+nj+k3";

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_TYPE);

		transliteration.setSourceToCompare(sourceToCompare);

		String[] res;
		try {
			res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}

			for (int i = 0; i < transliteration.getValue().size(); i++) {
				log.info("value(i) = " + transliteration.getValue().get(i));
				assertEquals(result2[i], transliteration.getValue().get(i));
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>INDICATOR_PPA + DEMONSTRATIVE_PRONOUN + DEMONSTRATIVE_PRONOUN +
	 * biliteral + biliteral + biliteral + triliteral.</b>
	 * </p>
	 * 
	 * @since 0.5
	 */
	@Test
	public void test9() {
		// initialize the data tests
		sequence = "jwnnnnwnh_nqnmt";

		result = new String[] {
				org.kemet.message.Messages.getString("INDICATOR_PPA"),
				org.kemet.message.Messages.getString("DEMONSTRATIVE_PRONOUN"),
				org.kemet.message.Messages.getString("DEMONSTRATIVE_PRONOUN"),
				org.kemet.message.Messages.getString("BILITERAL"),
				org.kemet.message.Messages.getString("BILITERAL"),
				org.kemet.message.Messages.getString("BILITERAL"),
				org.kemet.message.Messages.getString("TRILITERAL") };

		String[] result2 = new String[] { "jw", "nn", "nn", "wn", "h_n", "qn",
				"mt" };

		// simulation of dropped hieroglyphs
		String sourceToCompare = "jw+nn+nn+nj+wn+h_n+qn+mt";

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_TYPE);

		transliteration.setSourceToCompare(sourceToCompare);

		String[] res;
		try {
			res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}

			for (int i = 0; i < transliteration.getValue().size(); i++) {
				log.info("value(i) = " + transliteration.getValue().get(i));
				assertEquals(result2[i], transliteration.getValue().get(i));
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>INDICATOR_PPA + DEMONSTRATIVE_PRONOUN + DEMONSTRATIVE_PRONOUN +
	 * biliteral + biliteral.</b>
	 * </p>
	 * 
	 * @since 0.5
	 */
	@Test
	public void test10() {
		// initialize the data tests
		sequence = "jwnnnnwnh_n";

		result = new String[] {
				org.kemet.message.Messages.getString("INDICATOR_PPA"),
				org.kemet.message.Messages.getString("DEMONSTRATIVE_PRONOUN"),
				org.kemet.message.Messages.getString("DEMONSTRATIVE_PRONOUN"),
				org.kemet.message.Messages.getString("BILITERAL"),
				org.kemet.message.Messages.getString("BILITERAL") };
		String[] result2 = new String[] { "jw", "nn", "nn", "wn", "h_n" };

		// simulation of dropped hieroglyphs
		String sourceToCompare = "jw+nn+nn+wn+h_n";

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_TYPE);

		transliteration.setSourceToCompare(sourceToCompare);

		String[] res;
		try {
			res = transliteration.analyze();

			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}

			for (int i = 0; i < transliteration.getValue().size(); i++) {
				log.info("value(i) = " + transliteration.getValue().get(i));
				assertEquals(result2[i], transliteration.getValue().get(i));
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>uniliteral + biliteral.</b>
	 * </p>
	 * 
	 * @since 0.5
	 */
	@Test
	public void test11() {
		// initialize the data tests
		sequence = "3h_n";

		result = new String[] {
				org.kemet.message.Messages.getString("UNILITERAL"),
				org.kemet.message.Messages.getString("BILITERAL") };
		String[] result2 = new String[] { "3", "h_n" };

		// simulation of dropped hieroglyphs
		String sourceToCompare = "3+h_n";

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_TYPE);

		transliteration.setSourceToCompare(sourceToCompare);

		String[] res;
		try {
			res = transliteration.analyze();

			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}

			for (int i = 0; i < transliteration.getValue().size(); i++) {
				log.info("value(i) = " + transliteration.getValue().get(i));
				assertEquals(result2[i], transliteration.getValue().get(i));
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>uniliteral + uniliteral + uniliteral + biliteral + triliteral.</b>
	 * </p>
	 * 
	 * @since 0.5
	 */
	@Test
	public void test12() {
		// initialize the data tests
		sequence = "3mwh_nd3r";

		result = new String[] {
				org.kemet.message.Messages.getString("UNILITERAL"),
				org.kemet.message.Messages.getString("UNILITERAL"),
				org.kemet.message.Messages.getString("UNILITERAL"),
				org.kemet.message.Messages.getString("BILITERAL"),
				org.kemet.message.Messages.getString("TRILITERAL") };
		String[] result2 = new String[] { "3", "m", "w", "h_n", "d3r" };

		// simulation of dropped hieroglyphs
		String sourceToCompare = "3+m+w+h_n+d3r";

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_TYPE);

		transliteration.setSourceToCompare(sourceToCompare);

		String[] res;
		try {
			res = transliteration.analyze();

			if (res == null)
				assertTrue(false);
			else {
				assertEquals(res.length, result.length);

				if (res.length == result.length) {
					for (int i = 0; i < res.length; i++) {
						log.info("res(i) = " + res[i]);
						assertEquals(res[i], result[i]);
					}
				}

				for (int i = 0; i < transliteration.getValue().size(); i++) {
					log.info("value(i) = " + transliteration.getValue().get(i));
					assertEquals(result2[i], transliteration.getValue().get(i));
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>uniliteral + uniliteral + ideogram + biliteral + triliteral +
	 * triliteral.</b>
	 * </p>
	 * 
	 * @since 0.5
	 */
	@Test
	public void test13() {
		// initialize the data tests
		sequence = "mwjrh_nd3rjm3";

		result = new String[] {
				org.kemet.message.Messages.getString("UNILITERAL"),
				org.kemet.message.Messages.getString("UNILITERAL"),
				org.kemet.message.Messages.getString("IDEOGRAM"),
				org.kemet.message.Messages.getString("BILITERAL"),
				org.kemet.message.Messages.getString("TRILITERAL"),
				org.kemet.message.Messages.getString("TRILITERAL") };
		String[] result2 = new String[] { "m", "w", "jr", "h_n", "d3r", "jm3" };

		// simulation of dropped hieroglyphs
		String sourceToCompare = "m+w+jr+h_n+d3r+jm3";

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_TYPE);

		transliteration.setSourceToCompare(sourceToCompare);

		String[] res;
		try {
			res = transliteration.analyze();

			if (res == null)
				assertTrue(false);
			else {
				assertEquals(res.length, result.length);

				if (res.length == result.length) {
					for (int i = 0; i < res.length; i++) {
						log.info("res(i) = " + res[i]);
						assertEquals(res[i], result[i]);
					}
				}

				for (int i = 0; i < transliteration.getValue().size(); i++) {
					log.info("value(i) = " + transliteration.getValue().get(i));
					assertEquals(result2[i], transliteration.getValue().get(i));
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>uniliteral + biliteral + triliteral.</b>
	 * </p>
	 * 
	 * @since 0.5
	 */
	@Test
	public void test14() {
		// initialize the data tests
		sequence = "3h_nmt;mwt";

		result = new String[] {
				org.kemet.message.Messages.getString("UNILITERAL"),
				org.kemet.message.Messages.getString("BILITERAL"),
				org.kemet.message.Messages.getString("TRILITERAL") };
		String[] result2 = new String[] { "3", "h_n", "mt" };

		// simulation of dropped hieroglyphs
		String sourceToCompare = "3+h_n+mt;mwt";

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_TYPE);

		transliteration.setSourceToCompare(sourceToCompare);

		String[] res;
		try {
			res = transliteration.analyze();

			if (res == null)
				assertTrue(false);
			else {
				assertEquals(res.length, result.length);

				if (res.length == result.length) {
					for (int i = 0; i < res.length; i++) {
						log.info("res(i) = " + res[i]);
						assertEquals(res[i], result[i]);
					}
				}

				for (int i = 0; i < transliteration.getValue().size(); i++) {
					log.info("value(i) = " + transliteration.getValue().get(i));
					assertEquals(result2[i], transliteration.getValue().get(i));
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>noun + noun.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	public void test15() {
		// initialize the data tests
		sequence = "b3knh.t";

		result = new String[] { org.kemet.message.Messages.getString("NOUN"),
				org.kemet.message.Messages.getString("NOUN"), };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_TYPE);

		String[] res;
		try {
			res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>ideogram + feminine grammatical kind.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	public void test16() {
		// initialize the data tests
		sequence = "h.w.t";

		result = new String[] {
				org.kemet.message.Messages.getString("IDEOGRAM"),
				org.kemet.message.Messages
						.getString("FEMININE_GRAMMATICAL_KIND") };

		transliteration = new TransliterationManager(sequence, gm, phInfo,
				sortedPhInfo, TO_TYPE);

		String[] res;
		try {
			res = transliteration.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}
}
